use actix_web::{web, HttpResponse, Responder};
use sqlx::PgPool;

use crate::{endpoints::IdPath, utils::ApplicationError};
use brass_db::models::{Clothing, Role, User};

#[actix_web::delete("/clothing/{id}")]
pub async fn delete(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    path: web::Path<IdPath>,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin {
        return Err(ApplicationError::Unauthorized);
    }

    let Some(clothing) = Clothing::read(pool.get_ref(), path.id).await? else {
        return Ok(HttpResponse::NotFound().finish());
    };

    Clothing::delete(pool.get_ref(), clothing.id).await?;

    Ok(HttpResponse::Ok().finish())
}

#[cfg(test)]
mod tests {
    use crate::utils::test_helper::{test_delete, DbTestContext, RequestConfig, StatusCode};
    use brass_db::models::{Clothing, Role};
    use brass_macros::db_test;

    #[db_test]
    async fn deletes_clothing_fine_when_user_is_admin(context: &DbTestContext) {
        let app = context.app().await;

        Clothing::create(&context.db_pool, "Tuchuniform")
            .await
            .unwrap();
        assert_eq!(2, Clothing::read_all(&context.db_pool).await.unwrap().len());

        let config = RequestConfig::new("/clothing/1").with_role(Role::Admin);

        let response = test_delete(&context.db_pool, app, &config).await;
        assert_eq!(StatusCode::OK, response.status());

        assert_eq!(1, Clothing::read_all(&context.db_pool).await.unwrap().len());
        // TODO: reduce numbers by one when db migrations are joined together
    }

    #[db_test]
    async fn returns_unauthorized_when_user_is_user(context: &DbTestContext) {
        let app = context.app().await;
        let response = test_delete(&context.db_pool, app, &RequestConfig::new("/clothing/1")).await;

        assert_eq!(StatusCode::UNAUTHORIZED, response.status());
    }

    #[db_test]
    async fn returns_unauthorized_when_user_is_area_manager(context: &DbTestContext) {
        let app = context.app().await;
        let response = test_delete(
            &context.db_pool,
            app,
            &RequestConfig::new("/clothing/1").with_role(Role::AreaManager),
        )
        .await;

        assert_eq!(StatusCode::UNAUTHORIZED, response.status());
    }

    #[db_test]
    async fn returns_not_found_when_clothing_does_not_exist(context: &DbTestContext) {
        let app = context.app().await;
        let response = test_delete(
            &context.db_pool,
            app,
            &RequestConfig::new("/clothing/100").with_role(Role::Admin),
        )
        .await;

        assert_eq!(StatusCode::NOT_FOUND, response.status());
    }
}
